using Encompass;
using KavTest.Components;
using Microsoft.Xna.Framework;

namespace KavTest.Engines
{
    [Reads(typeof(TransformComponent), typeof(AngularVelocityComponent))]
    [Writes(typeof(TransformComponent))]
    public class RotationEngine : Engine
    {
        public override void Update(double dt)
        {
            foreach (var entity in ReadEntities<AngularVelocityComponent>())
            {
                if (HasComponent<TransformComponent>(entity))
                {
                    ref readonly var angularVelocityComponent = ref GetComponent<AngularVelocityComponent>(entity);
                    ref readonly var transformComponent = ref GetComponent<TransformComponent>(entity);

                    var angularVelocity = angularVelocityComponent.AngularVelocity * (float)dt;
                    var transform = transformComponent.Transform;

                    var newTransform = Matrix.Transform(transform, Quaternion.CreateFromYawPitchRoll(angularVelocity.X, angularVelocity.Y, angularVelocity.Z));
                    SetComponent(entity, new TransformComponent(newTransform));
                }
            }
        }
    }
}
